//
//  kp.c
//
//  Created by Mike Holenderski on 12/9/13.
//  Copyright (c) 2013 Mike. All rights reserved.
//

#include "kp.h"
#include "kp_protected.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int KPJoin(char* space, char* node, char* ip, int port) {
  ss_create(&ss_info, space, ip, port);

  if(ss_join(&ss_info, node) != -1) {
    ss_joined = &ss_info;
    return 0;
  } else {
    ss_joined = NULL;
    printf("Unable to join Smart Space with space=%s, node=%s, ip=%s, port=%d\n", space, node, ip, port);
    return -1;
  }
}

void KPLeave() {
  if (ss_joined) {
    ss_leave(ss_joined);
  }
}

bool KPStringToBool(char* str) {
  return !strcmp(str, "true");
}

int KPStringToInt(char* str) {
  return strtol(str, NULL, 10);
}

float KPStringToFloat(char* str) {
  return strtof(str, NULL);
}

int KPGetObjects(char* subject, char* predicate, KPUri* uris, int maxNumUris) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return 0; }

  ss_triple_t * sensor_rqst = NULL;
  ss_triple_t * first_triple = NULL;
  ss_triple_t * triple = NULL;
  int i;

  ss_add_triple(&sensor_rqst, subject, predicate, SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  if(ss_query(ss_joined, sensor_rqst, &first_triple) < 0) {
    printf("Unable to query objects with predicate %s\n", predicate);
    return -1;
  }

  ss_delete_triples(sensor_rqst);

  triple = first_triple;
  for(i = 0; i < maxNumUris && triple; i++) {
    strcpy(uris[i], triple->object);
    triple = triple->next;
  }

  ss_delete_triples(first_triple);

  return i;
}

void KPGetValue(char* subject, char* predicate, char* value) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  ss_triple_t * sensor_rqst = NULL;
  ss_triple_t * triple = NULL;

  ss_add_triple(&sensor_rqst, subject, predicate, SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  if(ss_query(ss_joined, sensor_rqst, &triple) < 0) {
    printf("Unable to query objects with predicate %s\n", predicate);
  }

  // retrieve the value
  if (triple) {
    strcpy(value, triple->object);
  } else {
    strcpy(value, ""); 
  }
  
  // clean up
  ss_delete_triples(sensor_rqst);
  ss_delete_triples(triple);
}

void KPAddObject(char* subject, char* predicate, char* uri) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  ss_bnode_t bnode;
  ss_bnode_t* bnodes = &bnode;
  
  ss_triple_t* triple = NULL;
  
  ss_add_triple(&triple, subject, predicate, SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_BNODE);
  ss_insert(ss_joined, triple, bnodes);
  ss_delete_triples(triple);
  
  strcpy(uri, bnodes->uri);
}

void KPUpdateObject(char* subject, char* predicate, char* object) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  ss_triple_t * insert_triples = NULL;
  ss_triple_t * removed_triples = NULL;

  ss_add_triple(&removed_triples, subject, predicate, SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&insert_triples, subject, predicate, object, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);

  ss_update(ss_joined, insert_triples, removed_triples, NULL);

  ss_delete_triples(insert_triples);
  ss_delete_triples(removed_triples);
}

void KPRemoveObject(char * uri) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  ss_triple_t * triples = NULL;

  ss_add_triple(&triples, SS_RDF_SIB_ANY, SS_RDF_SIB_ANY, uri, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&triples, uri, SS_RDF_SIB_ANY, SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);

  /* remove sensor from the smart space */
  if(ss_remove(ss_joined, triples) < 0) {
      printf("Unable to remove object with uri %s", uri);
  }

  ss_delete_triples(triples);
}
