//
//  kp_light.c
//  SmartKPI
//
//  Created by Mike on 14/9/13.
//  Copyright (c) 2013 Mike. All rights reserved.
//


#include "kp_light.h"
#include "kp_protected.h"

#include <stdio.h>
#include <string.h>

static char* KPLightTypeString[KPLightTypes] = {
  "discrete",
  "brightness"
};

char* KPLightTypeToString(KPLightType type) {
  return KPLightTypeString[type];
}

KPLightType KPStringToLightType(char* str) {
  KPLightType type;
  for (type = 0; type < KPLightTypes; type++) {
    if (!strcmp(str, KPLightTypeToString(type))) {
      break;
    }
  }
  if (type == KPLightTypes) {
    printf("Unknown light type %s", str);
    return -1;
  } else {
    return type;
  }
}

int KPGetLights(KPLight* lights, int maxLights) {
  KPUri uris[maxLights];
  int numLights = KPGetObjects(ss_info.space_id, "hasLight", uris, maxLights);
  
  for (int i = 0; i < numLights; i++) {
    KPGetLight(uris[i], &lights[i]);
  }

  return numLights;
}

void KPGetLight(KPUri uri, KPLight* light) {
  strcpy(light->uri, uri);
  KPGetValue(uri, "hasName", light->name);
  
  char type[kKPNameMaxLen];
  KPGetValue(uri, "isOfType", type);
  light->type = KPStringToLightType(type);
}

void KPAddLight(KPLightType type, char* name, char* location, char* uri) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  KPAddObject(ss_info.space_id, "hasLight", uri);

  ss_triple_t* triple = NULL;

  ss_add_triple(&triple, uri, "isOfType", KPLightTypeString[type], SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&triple, uri, "hasName", name, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&triple, uri, "hasLocation", location, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);

  ss_insert(ss_joined, triple, NULL);
  ss_delete_triples(triple);
}

void KPGetDiscreteLightValue(KPUri uri, KPDiscreteLightValue* value) {
  char v[kKPNameMaxLen];
  KPGetValue(uri, "isWarm", v);
  value->warm = KPStringToBool(v);

  KPGetValue(uri, "isBright", v);
  value->bright = KPStringToBool(v);

  KPGetValue(uri, "isDynamic", v);
  value->dynamic = KPStringToBool(v);
}

void KPGetBrightnessLightValue(KPUri uri, KPBrightnessLightValue* value) {
  char v[kKPNameMaxLen];
  
  KPGetValue(uri, "hasBrightness", v);
  value->brightness = KPStringToFloat(v);
}

void KPUpdateDiscreteLight(char* uri, KPDiscreteLightValue* value) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  ss_triple_t * insert_triples = NULL;
  ss_triple_t * removed_triples = NULL;

  ss_add_triple(&removed_triples, uri, "isWarm", SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&insert_triples, uri, "isWarm", value->warm ? "true" : "false", SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&removed_triples, uri, "isBright", SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&insert_triples, uri, "isBright", value->bright ? "true" : "false", SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&removed_triples, uri, "isDynamic", SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&insert_triples, uri, "isDynamic", value->dynamic ? "true" : "false", SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);

  ss_update(ss_joined, insert_triples, removed_triples, NULL);

  ss_delete_triples(insert_triples);
  ss_delete_triples(removed_triples);
}

void KPUpdateBrightnessLight(char* uri, KPBrightnessLightValue* value) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  ss_triple_t * insert_triples = NULL;
  ss_triple_t * removed_triples = NULL;
  
  char str[15];
  sprintf(str, "%f", value->brightness);

  ss_add_triple(&removed_triples, uri, "hasBrightness", SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&insert_triples, uri, "hasBrightness", str, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
 
  ss_update(ss_joined, insert_triples, removed_triples, NULL);

  ss_delete_triples(insert_triples);
  ss_delete_triples(removed_triples);
}

void KPFlashLight(char* lightUri) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  ss_triple_t * insert_triples = NULL;
  ss_add_triple(&insert_triples, lightUri, "flashForMilliseconds", "500", SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
 
  ss_insert(ss_joined, insert_triples, NULL);

  ss_delete_triples(insert_triples);
}

void KPRemoveLight(char* uri) {
  KPRemoveObject(uri);
}

#pragma mark Policy

int KPGetLightPolicies(KPLightPolicy* policies, int maxPolicies) {
  KPUri uris[maxPolicies];
  int numPolicies = KPGetObjects(ss_info.space_id, "hasPolicy", uris, maxPolicies);
  
  for (int i = 0; i < numPolicies; i++) {
    KPGetLightPolicy(uris[i], &policies[i]);
  }

  return numPolicies;
}

void KPGetLightPolicy(KPUri uri, KPLightPolicy* policy) {
  strcpy(policy->uri, uri);
  KPGetValue(uri, "hasName", policy->name);
  KPGetValue(uri, "isForUser", policy->user);
}

void KPAddLightPolicy(char* userUri, char* name, char* policyUri) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  KPAddObject(ss_info.space_id, "hasLightPolicy", policyUri);

  ss_triple_t* triple = NULL;
  ss_add_triple(&triple, policyUri, "isForUser", userUri, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&triple, policyUri, "hasName", name, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_insert(ss_joined, triple, NULL);
  ss_delete_triples(triple);
}


void KPRemoveLightPolicy(char* policyUri) {
  KPUri settingUris[kKPMaxLights];
  
  // remove all the policy settings
  int numSettings = KPGetObjects(policyUri, "hasLightSetting", settingUris, kKPMaxLights);
  for (int i = 0; i < numSettings; i++) {
    KPRemoveLightSetting(settingUris[i]);
  }
  
  // remove the policy itself
  KPRemoveObject(policyUri);
}

void KPEnactLightPolicy(char* userUri, char* policyUri) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  ss_triple_t * insert_triples = NULL;
  ss_triple_t * removed_triples = NULL;

  ss_add_triple(&removed_triples, userUri, "wantsLightPolicy", SS_RDF_SIB_ANY, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&insert_triples, userUri, "wantsLightPolicy", policyUri, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
 
  ss_update(ss_joined, insert_triples, removed_triples, NULL);

  ss_delete_triples(insert_triples);
  ss_delete_triples(removed_triples);
}

#pragma mark Setting

int KPGetLightPolicySettings(KPUri policyUri, KPLightPolicySetting* settings, int maxSettings) {
  KPUri settingUris[maxSettings];
  int numSettings = KPGetObjects(policyUri, "hasLightSetting", settingUris, maxSettings);
  
  for (int i = 0; i < numSettings; i++) {
    KPGetLightPolicySetting(settingUris[i], &settings[i]);
  }
  
  return numSettings;
}

void KPGetLightPolicySetting(KPUri uri, KPLightPolicySetting* setting) {
  strcpy(setting->uri, uri);

  KPGetValue(uri, "isForLight", setting->lightUri);
  
  char type[kKPNameMaxLen];
  KPGetValue(uri, "isOfType", type);
  setting->type = KPStringToLightType(type);
}

void KPAddLightPolicySetting(KPLightType type, char* policyUri, char* lightUri, char* settingUri) {
  if (!ss_joined) { printf("Trying to operate on smart space before joining one\n"); return; }

  KPAddObject(policyUri, "hasLightSetting", settingUri);

  ss_triple_t* triple = NULL;
  ss_add_triple(&triple, settingUri, "isForLight", lightUri, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&triple, settingUri, "isOfType", KPLightTypeString[type], SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_insert(ss_joined, triple, NULL);
  ss_delete_triples(triple);
}

void KPUpdateDiscreteLightSetting(char* settingUri, KPDiscreteLightValue* value) {
  KPUpdateDiscreteLight(settingUri, value);
}

void KPUpdateBrightnessLightSetting(char* settingUri, KPBrightnessLightValue* value) {
  KPUpdateBrightnessLight(settingUri, value);
}

void KPRemoveLightSetting(char* settingUri) {
  KPRemoveObject(settingUri);
}
