//
//  kp_light.h
//  SmartKPI
//
//  Created by Mike on 14/9/13.
//  Copyright (c) 2013 Mike. All rights reserved.
//

#ifndef kp_light_h
#define kp_light_h

#include "kp.h"

/*
 * Light device
 */

/* the max number of lights in a smar space */
#define kKPMaxLights 10

typedef enum {
  KPLightTypeDiscrete,
  KPLightTypeBrightness,
  KPLightTypes
} KPLightType;

typedef struct {
  KPUri uri;
  char name[kKPNameMaxLen];
  KPLightType type;
} KPLight;

typedef struct {
  bool warm;
  bool bright;
  bool dynamic;
} KPDiscreteLightValue;

typedef struct {
  float brightness;
} KPBrightnessLightValue;

char* KPLightTypeToString(KPLightType type);

int KPGetLights(KPLight* lights, int maxLights);
void KPGetLight(KPUri uri, KPLight* light);

void KPAddLight(KPLightType type, char* name, char* location, char* uri);

void KPGetDiscreteLightValue(KPUri uri, KPDiscreteLightValue* value);
void KPUpdateDiscreteLight(char* lightUri, KPDiscreteLightValue* value);

void KPGetBrightnessLightValue(KPUri uri, KPBrightnessLightValue* value);
void KPUpdateBrightnessLight(char* lightUri, KPBrightnessLightValue* value);

void KPFlashLight(char* lightUri);

void KPRemoveLight(char* lightUri);

/*
 * Light policy
 */

/* the max number of policies in a smar space */
#define kKPMaxLightPolicies 10
#define kKPMaxLightPolicySettings kKPMaxLights

typedef struct {
  KPUri uri;
  char name[kKPNameMaxLen];
  char user[kKPNameMaxLen];
} KPLightPolicy;

typedef struct {
  KPUri uri;
  KPUri lightUri;
  KPLightType type;
} KPLightPolicySetting;

int KPGetLightPolicies(KPLightPolicy* policies, int maxPolicies);
void KPGetLightPolicy(KPUri uri, KPLightPolicy* policy);

void KPAddLightPolicy(char* userUri, char* name, char* policyUri);
void KPRemoveLightPolicy(char* policyUri);
void KPEnactLightPolicy(char* userUri, char* policyUri);

int KPGetLightPolicySettings(KPUri policyUri, KPLightPolicySetting* settings, int maxSettings);
void KPGetLightPolicySetting(KPUri uri, KPLightPolicySetting* setting);

void KPAddLightPolicySetting(KPLightType type, char* policyUri, char* lightUri, char* settingUri);
void KPUpdateDiscreteLightSetting(char* settingUri, KPDiscreteLightValue* value);
void KPUpdateBrightnessLightSetting(char* settingUri, KPBrightnessLightValue* value);
void KPRemoveLightSetting(char* settingUri);

#endif /* kp_light_h */ 
